require 'hashie/mash'

module Hyrb
  class Command < Thor
    def self.subc(name, description)
      desc name.to_sym, description
      subcommand name.to_s, "hyrb/commands/#{name}".camelcase.constantize
    end

    no_commands do
      def pipeline(task, env = {})
        env.merge!(verbose: !!options[:verbose])
        Commands::Pipeline.new(task).run(Hashie::Mash.new(env))
      end
    end
  end

  module Commands
    Hyrb.autoload_each self, "hyrb/commands", %w( ansible creds defaults developers digital_ocean environment github hipchat project provision rackspace )

    class Pipeline < Hyrb::Pipeline
      def initialize(task_class)
        super

        # Thor:Shell defines a constructor so have to do this here
        extend Thor::Shell
      end

      def prompt(prompt, hash, key, options = {})
        unless options.is_a?(Hash)
          default = options
          options = {}
          options[:default] = default
        end

        if options.delete(:always)
          options[:default] = hash[key] if hash[key]
          hash[key] = ask(prompt, options)
        else
          hash[key].blank? ? hash[key] = ask(prompt, options) : hash[key]
        end

        hash.save! if hash.respond_to? :save!
      end

      def edit(path)
        editor = [ENV['VISUAL'], ENV['EDITOR']].find { |e| e.present? }
        raise "Please set $EDITOR variable :)" unless editor
        cmd = "#{editor} #{path}"
        raise "Could not run '#{cmd}'" unless system(cmd)
      end

      def option_list(cache, attribute = nil, default = nil, &block)
        list = attribute ? cache[attribute] : cache
        choices = list.map.with_index(&block)
        say choices.join("\n")
        index = ask("Pick from #{attribute}:", default: default)
        if index.try :include?, ","
          index.split(",").map { |i| list[i.to_i - 1] }
        else
          list[index.to_i - 1]
        end
      end

      def info(message)
        say message
      end

      def beep
        print "\a"
      end
    end
  end
end