require 'yaml'
require 'hashie/mash'

module Hyrb
  module Models
    Hyrb.autoload_each self, "hyrb/models", %w( ansible_host ansible_site cache creds defaults developer environment project )
  end

  class Model
    extend Forwardable
    include Enumerable

    attr_accessor :path, :data

    def_delegators :data, :each

    def initialize(path, data = nil)
      @path = File.join(ENV["HYFN_CONFIG_PATH"] || Hyrb::DEFAULT_PATH, path)
      @data = deserialize(load_config_file) || data
    end

    def load_config_file
      YAML.load_file(filepath)
    rescue Errno::ENOENT
      nil
    end

    def filepath
      @path + ".yml"
    end

    def reload!
      @data = self.load_config_file
    end

    def save!
      FileUtils.mkdir_p(File.dirname(@path))
      File.open(filepath, File::RDWR|File::TRUNC|File::CREAT, 0600) do |file|
        file.write(serialize(@data).to_yaml)
      end
    end

    protected

    def serialize(data)
      data
    end

    def deserialize(data)
      data
    end
  end

  module Models
    class Struct < Model
      def self.keys
        @keys
      end

      def self.define_keys(keys)
        @keys = keys.each do |key|
          define_method key do
            @data[key]
          end

          define_method "#{key}=" do |val|
            @data[key] = val
          end
        end
      end

      def initialize(path)
        super
        self.class.keys.each { |key| @data[key] ||= nil }
      end

      def [](key)
        @data[key]
      end

      def []=(key, val)
        @data[key] = val
      end

      protected

      def serialize(data)
        data.to_hash
      end

      def deserialize(data)
        Hashie::Mash.new(data)
      end
    end
  end
end