module Hyrb
  module Models
    class AnsibleSite < Struct
      attr_accessor :vars_prompt, :roles, :hosts

      define_keys %w( project_name
                      host
                      deploy_user
                      base_path
                      domain
                      relative_web_root
                      mysql_host
                      mysql_db
                      mysql_user
                      mysql_password )
  
      def initialize(project, environment)
        @path = File.join(project.ansible_path, environment.label)
        @raw_playbook = load_config_file[0]
        @data = deserialize(@raw_playbook)

        @vars_prompt = @raw_playbook["vars_prompt"]
        @roles       = @raw_playbook["roles"]
        @hosts       = @raw_playbook["hosts"]
      end

      def load_config_file
        super || YAML.load_file("#{TEMPLATE_PATH}/ansible/site.yml")
      end

      def serialize(data)
        # yuck, this is no longer a pure fn
        @raw_playbook["vars_prompt"] = vars_prompt
        @raw_playbook["roles"]       = roles
        @raw_playbook["hosts"]       = hosts
        
        @raw_playbook["vars"] = @data.map { |k, v| {k.to_s => v} }
        [@raw_playbook]
      end

      def deserialize(data)
        data["vars"].reduce(Hash.new, :merge)
      end
    end
  end
end