require 'tsort'

module Hyrb
  class Pipeline
    include TSort

    attr_accessor :prev_tasks, :next_tasks, :current_task, :env

    def self.rules
      @@rules ||= {}
    end

    def initialize(task_class)
      @task_class = task_class
      @prev_tasks = []
      @next_tasks = build_task_list
    end

    def run(env = {})
      @env = env
      while @current_task = @next_tasks.shift
        task = @current_task.new(self)
        info "--> Running #{@current_task}" if @env[:verbose]
        task.run_before(@env)
        task.run(@env)
        @prev_tasks << @current_task
      end
      @env
    end

    def invoke(task_class)
      pipeline = self.class.new(task_class)
      pipeline.prev_tasks += self.prev_tasks
      pipeline.next_tasks -= self.prev_tasks
      pipeline.run(@env)
      self.next_tasks -= pipeline.prev_tasks
    end

    def info(message)
      puts message
    end

    def tsort_each_child(node, &block)
      (self.class.rules[node] || []).each(&block)
    end

    def tsort_each_node(&block)
      (self.class.rules || {}).each_key(&block)
    end

    def build_task_list(id_map={}, stack=[])
      arr = []
      each_strongly_connected_component_from(@task_class, id_map, stack) do |t|
        if t.length == 1
          arr << t.first
        else
          raise TSort::Cyclic.new("cyclic dependencies: #{t.inspect}")
        end
      end
      arr
    end
  end
end