module Hyrb
  module Tasks
    module Environment
      class Inject < Task
        depends Project::Init

        def run(env)
          unless env.environment_name
            if env.project.environments.any?
              say "Environments: #{env.project.environments.keys.join(", ")}"
            end

            env.environment_name = ask "Environment", default: "production"
          end

          env.environment = Hyrb::Models::Environment.new(env.project, env.environment_name)
          env.environment.label = env.project.name
          env.environment.label += "-#{env.environment_name}" unless env.environment_name == "production"

          prompt "Domain (FQDN)", env.environment, :domain, "#{env.environment.label}.#{env.defaults.domain}"

          env.environment.save!
        end
      end

      class Init < Task
        depends Inject
      end

      class SetupExisting < Task
        depends Init

        prompt :environment, :host, default: ->(env) { env.environment.domain }
      end

      class Deployment < Task
        depends SetupExisting

        prompt :environment, :deploy_user, default: "deploy"
        prompt :environment, :base_path, default: "/var/www"
        prompt :environment, :relative_web_root, default: "/current/public"

        def run(env)
        end
      end

      class Database < Task
        depends Deployment

        prompt :environment, :database_host, default: "localhost"
        prompt :environment, :database_name, default: ->(env) { env.environment.label.underscore }
        prompt :environment, :database_user, default: ->(env) { env.environment.label.underscore }
      end
    end
  end
end