require 'hipchat-api'
  
module Hyrb
  module Tasks
    module Hipchat
      class Inject < Task
        depends Creds::Init

        prompt :creds, :hipchat_api_key

        def run(env)
          env.hipchat_client = HipChat::API.new(env.creds.hipchat_api_key)
        end
      end

      class Init < Task
        depends Inject
      end

      class Rooms < Task
        depends Init

        def run(env)
          env.hipchat_rooms = env.hipchat_client.rooms_list['rooms']
        end
      end

      class ShowRooms < Task
        depends Rooms

        def run(env)
          say env.hipchat_rooms.map { |r| "#{r.name} (#{r.room_id})" }.to_yaml
        end
      end

      class CreateRoom < Task
        depends Rooms, Project::Init

        prompt :project, :room_name, default: ->(env) { env.project.title }
        prompt :creds, :hipchat_user_id

        def run(env)
          # TODO: should unarchive room if it exists
          if env.hipchat_rooms.any? { |r| r.name == env.project.room_name }
            say "HipChat room #{env.project.room_name} exists", :yellow
          else
            env.hipchat_client.rooms_create(env.project.room_name, env.creds.hipchat_user_id,
                                            'public', env.project.description)
            say "Created HipChat room #{env.project.room_name}", :green
          end
        end
      end

      class ArchiveOldRooms < Task
        depends Rooms

        def run(env)
          floor = Time.now - 3.months
          say "Listing all unarchived room last active before #{floor}"
          say "CTRL+C to exit"
          old_rooms = env.hipchat_rooms.select do |room|
            room["is_archived"] == false && room["last_active"] < floor.to_i
          end

          old_rooms.each do |room|
            if yes? "Archive #{room["name"]}? (last active at #{Time.at(room["last_active"])})"
              say "Hipchat does not support archiving rooms via the API :(", :red
            else
              say "Spared #{room["name"]}", :yellow
            end
          end
        end
      end
    end
  end
end