require 'fog/digitalocean'
require 'fog/rackspace'
require 'parallel'

module Hyrb
  module Tasks
    module Provision
      class Init < Task
        depends Environment::Init

        prompt :environment, :server_name, default: ->(env) { env.environment.domain || env.environment.label }
      end

      class DigitalOcean < Task
        depends Init, Hyrb::Tasks::DigitalOcean::SSHKeys, Developers::AddToProject

        def run(env)
          ssh_key_ids = env.project.developers(env.developers).map(&:digital_ocean_id).compact

          flavor_id = option_list(env.digital_ocean_cache, :flavors, 2) do |f, i|
            "#{i+1}: #{f.name} ram / #{f.cpu} cpu / #{'$%.2f/mo' % f.cost_per_month}"
          end.id

          image_id = option_list(env.digital_ocean_cache, :images, 16) do |f, i|
            "#{i+1}: #{f.name} / id: #{f.id}"
          end.id

          region_id = option_list(env.digital_ocean_cache, :regions, 2) do |f, i|
            "#{i+1}: #{f.name} / id: #{f.id}"
          end.id

          say "Creating Digital Ocean Server"
          server = env.digital_ocean_client.servers.create({
            name: env.environment.server_name,
            flavor_id: flavor_id,
            image_id: image_id,
            region_id: region_id,
            ssh_key_ids: ssh_key_ids,
          })

          env.environment.provider = 'digital_ocean'
          env.environment.server_id = server.id

          sleep(4) until server.reload.ready?
          env.environment.host = server.public_ip_address

          env.environment.save!

          say "Created Digital Ocean Server!", :green
          say "IP Address: #{server.public_ip_address} / ID: #{server.id}", :green
          say "\n\tssh root@#{server.public_ip_address}\n"
          beep
        end
      end

      class Rackspace < Task
        depends Init, Hyrb::Tasks::Rackspace::Init

        def run(env)
          flavor_id = option_list(env.rackspace_cache, :flavors, 2) do |f, i|
            "#{i+1}: #{f.name} / id: #{f.id}"
          end.id

          image_id = option_list(env.rackspace_cache, :images, 1) do |f, i|
            "#{i+1}: #{f.name} / id: #{f.id}"
          end.id

          say "Creating Rackspace Server"
          server = env.rackspace_client.servers.create({
            name: env.environment.server_name,
            flavor_id: flavor_id,
            image_id: image_id,
          })

          env.environment.provider = 'rackspace'
          env.environment.server_id = server.id

          sleep(6) until server.reload.ready?
          env.environment.host = server.public_ip_address

          env.environment.save!

          say "Created Rackspace Server!", :green
          say "IP Address: #{server.public_ip_address} / ID: #{server.id}", :green
          say "\n\tssh root@#{server.public_ip_address}\n"
          beep
        end
      end
    end
  end
end